@@ -76,16 +76,15 @@ class CollectionItemClass(object):
     def __getattr__(self, __name: str, *a, **kw):
       return self._item.__getattribute__(__name, *a, **kw)
 
-
-
 class QueryResult(object):
     def __init__(self, cursor, pager, data_mapper=None):
         self.cursor = cursor
         stats = cursor.statistics()
-        current_count = self.cursor.count()
-        self.count = stats["fullCount"]
-        self.pagination = lib.gen_pagination(total_count=self.count,
-                                            count=current_count,
+        self.count = self.cursor.count() # current count
+        self.size = stats["fullCount"] # total count
+        self.pagination = lib.gen_pagination(
+                                            size=self.size,
+                                            count=self.count,
                                             page=pager[0],
                                             per_page=pager[1])
 
@@ -98,7 +97,9 @@ class QueryResult(object):
             yield self._data_mapper(item)
 
     def __len__(self):
-        return self.count
+        """Get the total count """
+        return self.size
+
 
 class Item_Impl(dict):
     NAMESPACE = None
@@ -352,8 +353,8 @@ class Item_Impl(dict):
         oplog = self._update({op: True})
         return oplog.get(op)
 
-    def currdate(self, path:str, value:Any=True):
-        op = "%s:$currdate" % self._make_path(path)
+    def timestamp(self, path:str, value:Any=True):
+        op = "%s:$timestamp" % self._make_path(path)
         oplog = self._update({op: value})
         return oplog.get(op)   
 
@@ -427,7 +428,6 @@ class CollectionItem(Item_Impl):
     def set_immut_keys(self, immut_keys:list=[]):
         self._immut_keys = immut_keys
 
-
     @contextmanager
     def context(self):
         """
@@ -519,7 +519,7 @@ class CollectionItem(Item_Impl):
     @property
     def subcollections(self) -> list:
         """ List all collections """
-        return list(self._subcollections.keys()) or []
+        return list(self._subcollections.keys())
 
     def drop_subcollection(self, name: str):
         try:
@@ -605,11 +605,19 @@ class CollectionItem(Item_Impl):
             self
         """
         if isinstance(nattime, str):
-            self.update({"__ttl:$currdate": nattime})
+            self.update({"__ttl:$timestamp": nattime})
         elif nattime is False:
             self.update({"__ttl": None})
         return self
 
+    def delete(self) -> bool:
+        """
+        To delete an item
+        Todo: deleted links/edges
+        """
+        self._collection.delete(self._key)
+        return True
+
 class SubCollection(object):
     _data = []
     _constraints = []
@@ -683,7 +691,7 @@ class SubCollection(object):
             data["_key"] = _key
         item = data
 
-        item = _create_document_item(data)
+        item = _create_subdocument_item(data)
         self._data.append(item)
         self._commit()
         return SubCollectionItem(self, item)
@@ -736,7 +744,7 @@ class SubCollection(object):
         """
         return self.find_one({"_key": _key})
 
-    def find_one(self, filters:dict={}):
+    def find_one(self, filters:dict={}) -> "SubCollectionItem":
         """
         Return only one item by criteria
 
@@ -747,7 +755,7 @@ class SubCollection(object):
             return list(res)[0]
         return None 
 
-    def find(self, filters: dict = {}, sorts: dict = {}, limit: int = 10, skip: int = 0) -> dict_query.Cursor:
+    def find(self, filters: dict = {}, sorts: dict = {}, limit: int = 10,  offset:int=0) -> dict_query.Cursor:
         """
         Perform a query
 
@@ -755,11 +763,11 @@ class SubCollection(object):
             filters:
             sorts:
             limit:
-            skip:
+            offset:
         """
         sorts = _parse_sort_dict(sorts, False)
         data = [SubCollectionItem(self, d) for d in dict_query.query(data=self._data, filters=filters)]
-        return dict_query.Cursor(data, sort=sorts, limit=limit, skip=skip)
+        return dict_query.Cursor(data, sort=sorts, limit=limit, offset=offset)
 
     def filter(self, filters: dict = {}) -> dict_query.Cursor:
         """
@@ -797,6 +805,10 @@ class SubCollectionItem(Item_Impl):
         self._key = data.get("_key")
         super().__init__(data)
 
+    def delete(self):
+        self._subcollection.delete({"_key": self._key})
+        return True
+        
 #------------------------------------------------------------------------------
 
 class Database(object):
@@ -811,7 +823,7 @@ class Database(object):
                  password:str=None, 
                  dbname: str = SYSTEM_DB, 
                  client:"Database"= None, 
-                 default_indexes:dict={},
+                 default_indexes:list=[],
                  query_max_limit=100,
                  collection_prefix:str=None,
                  custom_ops:dict={}):
@@ -823,9 +835,9 @@ class Database(object):
             password
             dbname
             client:Database
-            default_indexes:dict
+            default_indexes:list
             query_max_limit
-            collection_prefix:str - a prefix to add in all collection name
+            collection_prefix:str|function - a prefix to add in all collection name
             custom_ops:dict - 
         
         """
@@ -852,7 +864,10 @@ class Database(object):
 
     def _prefix_collection_name(self, collection_name:str) -> str:
         if self._collection_prefix:
-            collection_name = "%s%s" % (self._collection_prefix, collection_name)
+            if isinstance(self._collection_prefix, str):
+                collection_name = "%s%s" % (self._collection_prefix, collection_name)
+            elif callable(self._collection_prefix):
+                collection_name = self._collection_prefix(collection_name)
         return collection_name
 
     def has_db(self, dbname:str=None) -> bool:
@@ -907,7 +922,6 @@ class Database(object):
     def has_collection(self, collection_name) -> bool:
         """
         Test if collection exists in the current db. 
-        *Must pass with the prefix, if needed
 
         Params:
             collection_name:str - the collection name 
@@ -915,9 +929,29 @@ class Database(object):
         Returns:
             bool
         """
+        collection_name = self._prefix_collection_name(collection_name)
         return self.db.has_collection(collection_name)
 
-    def select_collection(self, collection_name:str, indexes=None, immut_keys=None, user_defined=True, item_class=None) -> "Collection":
+    def create_collection(self, collection_name:str, indexes:list=[]) -> bool:
+        """
+        Create a collection if not exists
+        Returns: bool
+        """
+        if not self.has_collection(collection_name):
+            collection_name = self._prefix_collection_name(collection_name)
+            col = self.db.create_collection(collection_name)
+
+            # indexes
+            if not indexes:
+                indexes = self.default_indexes
+
+            if indexes and isinstance(indexes, list):
+                for index in [*indexes, *DEFAULT_INDEXES]:
+                    col._add_index(index) 
+            return True 
+        return False
+
+    def select_collection(self, collection_name:str, indexes:list=[], immut_keys:list=[], item_class=None, auto_create:bool=True) -> "Collection":
         """
         To select a collection
 
@@ -925,32 +959,30 @@ class Database(object):
             collection_name:str - collectioin name 
             indexes:List[dict] - the indexes to use
             immut_keys:list - immutable keys. Keys that can't be updated once created
+            auto_create:bool - To auto create the collection if doesn't exist
 
-        Return:
-            Collection
+        Return: Collection
 
         """
 
-        collection_name = self._prefix_collection_name(collection_name)
         if self.has_collection(collection_name):
+            collection_name = self._prefix_collection_name(collection_name)
             col = self.db.collection(collection_name)
+        elif auto_create is True:
+            self.create_collection(collection_name=collection_name, indexes=indexes)
+            collection_name = self._prefix_collection_name(collection_name)
+            col = self.dbcollection(collection_name)
         else:
-            col = self.db.create_collection(collection_name)
-            if not indexes and user_defined is True and self.default_indexes:
-                indexes = self.default_indexes
-            
-            # indexes
-            if indexes and isinstance(indexes, list):
-                for index in [*indexes, *DEFAULT_INDEXES]:
-                    col._add_index(index) 
+            raise CollectionNotFoundError()
 
         return Collection(db=self, collection=col, immut_keys=immut_keys, custom_ops=self._custom_ops, item_class=item_class)
 
     def select_edge_collection(self, collection_name:str):
-        collection_name = self._prefix_collection_name(collection_name)
         if self.db.has_collection(collection_name):
+            collection_name = self._prefix_collection_name(collection_name)
             return self.db.collection(collection_name)
         else:
+            collection_name = self._prefix_collection_name(collection_name)
             return self.db.create_collection(name=collection_name, edge=True)
 
     def get_item(self, path:str) -> CollectionItem:
@@ -1014,13 +1046,14 @@ class Database(object):
             kvmap:dict
             data_mapper:function - a callback function
         Returns
-            tuple(cursor:ArangoCursor, pagination:dict)
+            QueryResult
         """
-        aql, bind_vars, pager = self.build_query(xql=xql, data=data, kvmap=kvmap, parser=parser)
+
+        aql, bind_vars, pager = self._build_query(xql=xql, data=data, kvmap=kvmap, parser=parser)
         cursor = self.execute_aql(aql, bind_vars=bind_vars, count=True, full_count=True)            
         return QueryResult(cursor=cursor, pager=pager, data_mapper=data_mapper)
 
-    def build_query(self, xql:lib_xql.XQLDEFINITION, data:dict={}, kvmap:dict={}, parser=None):
+    def _build_query(self, xql:lib_xql.XQLDEFINITION, data:dict={}, kvmap:dict={}, parser=None):
         """
         Build a query from XQL
 
@@ -1032,15 +1065,15 @@ class Database(object):
         """        
         xql = lib_xql.prepare_xql(xql)
         # replace the kvmap
-        xql["FILTER"] = lib.dict_find_replace(xql["FILTER"], kvmap)
+        xql["FILTERS"] = lib.dict_find_replace(xql["FILTERS"], kvmap)
 
         # pagination
         if "page" in data:
             xql["PAGE"] = data.get("page") or 1
             del data["page"]
-        if "take" in data:
-            xql["TAKE"] = data.get("take") or 10
-            del data["take"]
+        if "limit" in data:
+            xql["LIMIT"] = data.get("limit") or 10
+            del data["limit"]
 
         _per_page, _, _page = lib_xql.xql_take_skip_page(xql=xql, max_limit=self.query_max_limit)
         aql, bind_vars = lib_xql.xql_to_aql(xql, vars=data, parser=parser, max_limit=self.query_max_limit)
@@ -1068,8 +1101,8 @@ class Database(object):
             return self.select(new_name)
     
     def drop_collection(self, collection_name:str):
-        collection_name = self._prefix_collection_name(collection_name)
         if self.has_collection(collection_name):
+            collection_name = self._prefix_collection_name(collection_name)
             self.db.delete_collection(collection_name)
 
     def add_index(self, collection_name, data:dict):
@@ -1240,15 +1273,22 @@ class Collection(object):
         self.collection_name = self.collection.name
         self.item_class = item_class
 
-    def _commit(self, item:CollectionItem):
+    def _commit(self, item:CollectionItem, replace_document=False):
         """
-        Save the item in the db
+        To commit/save changes
+
+        Params:
+            item:CollectionItem
+            replace_document:bool - To replace instead of updating. Replace will not merge the data.
         """
         if not item._key:
             raise MissingItemKeyError()
         try:
-            item.currdate('_modified_at')
-            return self.collection.update(item.to_dict(), return_new=True)["new"]
+            if replace_document:
+                return self.collection.replace(item.to_dict(), return_new=True)["new"]
+            else:
+                item.timestamp('_modified_at')
+                return self.collection.update(item.to_dict(), return_new=True)["new"]
         except DocumentUpdateError as due:
             item.update({"_modified_at": None})
             return self.collection.insert(item.to_dict(), return_new=True)["new"]
@@ -1330,16 +1370,21 @@ class Collection(object):
         self.collection.insert(item.to_dict(), silent=True)
         return self.get(item._key) 
 
-    def update(self, _key:str, data:dict) -> CollectionItem:
+    def update(self, _key:str, data:dict, replace_document=False) -> CollectionItem:
         """
-        Update an item
+        To update and item. Can also replace the item
+
+        Params:
+            _key:str - document key 
+            data:dict - data to update
+            replace_document:bool - If true will replace the document and not merge
 
         Returns
             CollectionItem
 
         """
         item = self.item({**data, "_key": _key})
-        self._commit(item)  
+        self._commit(item, replace_document=replace_document)  
         return self.get(item._key) 
 
     def upsert(self, data:dict) -> CollectionItem:
@@ -1365,7 +1410,7 @@ class Collection(object):
         """
         self.collection.delete(_key)
 
-    def find(self, filters:dict={}, skip=None, limit=10, sort=None, page=None):
+    def find(self, filters:dict={}, offset=None, limit=10, sort=None, page=None):
         """
         Perform a find in the collections
 
@@ -1373,14 +1418,15 @@ class Collection(object):
             Generator[CollectionItem]
         """
         
-        if skip is None and page:
-            skip = lib.calc_pagination_offset(page=page, per_page=limit)
+
+        if offset is None and page:
+            offset = lib.calc_pagination_offset(page=page, per_page=limit)
 
         xql = {
             "FROM": self.collection_name, 
-            "FILTER": filters,
-            "SKIP": skip,
-            "TAKE": limit,
+            "FILTERS": filters,
+            "OFFSET": offset,
+            "LIMIT": limit,
             "SORT": sort
         }
 
@@ -1423,12 +1469,22 @@ def _create_document_item(data:dict={}) -> dict:
     _key = data["_key"] if "_key" in data else lib.gen_key()
 
     return {
-        **data,
         "_key": _key,
-        "_created_at:$currdate": True,
-        "_modified_at": None
+        "_created_at:$timestamp": True,
+        "_modified_at": None,
+        "__ttl": None,
+        **data,
     }
 
+def _create_subdocument_item(data:dict={}) -> dict:
+    _key = data["_key"] if "_key" in data else lib.gen_key()
+
+    return {
+        "_key": _key,
+        "_created_at:$timestamp": True,
+        "_modified_at": None,
+        **data,
+    }
 
 def _ascdesc(v, as_str=True):
     if as_str:
